from src.http_invoker import HttpInvoker
from falcon import Request, Response  # type: ignore
from typing import Any

class FalconHttpInvoker(HttpInvoker):  # type: ignore
    def __init__(self, invocable: Any):
        HttpInvoker.__init__(self, invocable)

        self.add_route(self.route, self)

    def on_get(self, request: Request, response: Response) -> None:
        data_out: List[Any] = []
        data_in: List[Any] = request.params
        # data_in(f'route: {str(request.url_rule)}')
        try:
            self._invocable.invoke(data_in, data_out)
        except Exception as err:
            abort(400)

        response.body = str(data_out)



from flask import Flask, request, abort
from src.http_invoker import HttpInvoker

class FlaskHttpInvoker(HttpInvoker):
    def start(self) -> int:
        app = Flask(__name__)

        @app.route(self.route)
        def handler() -> str:
            data_out: List[Any] = []
            data_in: List[Any] = request.args
            # data_in(f'route: {str(request.url_rule)}')
            try:
                self._invocable.invoke(data_in, data_out)
            except Exception as err:
                print(err)
                abort(400)

            return str(data_out)

        app.run()
        return 0



import click
import json
from typing import List, Optional
from src.flask_http_invoker import FlaskHttpInvoker
from src.function_invocable import FunctionInvocable
import subprocess
import re

@click.group()
def main():
    pass

@main.command()
@click.argument('reference', type=click.STRING) # a function referenced by <module>[.<class>][:<function>]
def stdio(reference: str):
    pass

@main.command()
@click.argument('reference', type=click.STRING) # a function referenced by <module>[.<class>][:<function>]
@click.argument('argv', nargs=-1)
def run(reference: str, argv: List):
    host = FunctionInvocable(reference)
    result = []
    host.invoke(argv, result) # 32.4
    click.echo(str(result))

@main.command()
@click.argument('reference', type=click.STRING) # a function referenced by <module>[.<class>][:<function>]
def http(reference: str):
    host = FlaskHttpInvoker(FunctionInvocable(reference))
    host.start()


@main.command()
@click.argument('scope', type=click.STRING, required=False) # major, minor, patch
def version(scope: Optional[str] = None):
    tag = None
    try:
        tag = subprocess.check_output(["git", "describe", "--tags"]).decode('ascii').strip()
    except subprocess.CalledProcessError:
        print('no previous version')
        exit(0)

    pattern: str = r'([0-9]+)\.([0-9]+)\.([0-9]+)(\-alpha)?(\-.*)?'
    matches = re.match(pattern, tag)

    if not scope:
        print(f'{matches.group(1)}.{matches.group(2)}.{matches.group(3)}{matches.group(4)}')
        exit(0)


    verdict: Dict[str, [int, str]] = {
        'major': int(matches.group(1)),
        'minor': int(matches.group(2)),
        'patch': int(matches.group(3)),
        'build': matches.group(4)
    }
    verdict[scope] += 1

    new_version = f'{verdict["major"]}.{verdict["minor"]}.{str(verdict["patch"]).zfill(2)}{verdict["build"]}'
    subprocess.check_call(["git", "tag", new_version])
    subprocess.check_call(["git", "push", "--tags"])



import importlib.util
import os
import re
import sys
from importlib.abc import Loader
from typing import Any


class FunctionInvocable(object):
    def __init__(self, reference: str) -> None:
        self._func: Any = None
        self._reference: str = reference
        self.inject()

    @property
    def func(self) -> Any:
        return self._func

    @func.setter
    def func(self, arg: Any) -> None:
        self._func = arg

    def invoke(self, data_in: Any, data_out: Any) -> None:
        result: Any = None

        try:
            result = self._func(*data_in)
        except Exception as err:
            raise Exception(f'Referenced function {self._reference} threw an exception: {str(err)}')

        data_out.append(result)

    def inject(self,) -> None:
        # [path/to/file/[file.extension[:[class.]method]]]
        pattern = r'^(.*\/)?([^\.\/]+)\.([^\.]+):([^:]+\.)?([^:\.]+)$' # (path/to/file/)(file).(extension):(method)
        matches = re.match(pattern, self._reference)
        if not matches:
            raise Exception(f'Unable to inject invalid referenced function {self._reference}')

        path_to_source_file = matches.group(1)
        if not matches.group(1):
            path_to_source_file = os.getcwd()
        elif matches.group(1)[0] != '/':
            path_to_source_file = f'{os.getcwd()}/{matches.group(1)}'
        source_file_name = matches.group(2)
        source_file_extension = matches.group(3)
        sys.path.insert(0, path_to_source_file)

        spec = importlib.util.spec_from_file_location(source_file_name, f'{path_to_source_file}/{source_file_name}.{source_file_extension}')
        module = importlib.util.module_from_spec(spec)
        assert isinstance(spec.loader, Loader)  # see https://github.com/python/typeshed/issues/2793
        spec.loader.exec_module(module)

        scope = module
        if matches.group(4):
            class_name = matches.group(4)[:-1]
            scope = getattr(scope, class_name)

        method_name = matches.group(5)
        self._func = getattr(scope, method_name)





from typing import Dict

class HttpInvoker(object):
    def __init__(self, invocable: 'HttpInvoker') -> None:
        super().__init__()
        self._invocable = invocable
        self._route: str = '/multiply'
        self._port: int = 8080

    @property
    def route(self) -> str:
        return self._route

    @route.setter
    def route(self, arg: str) -> None:
        self._route = arg

    @property
    def port(self) -> int:
        return self._port

    @port.setter
    def port(self, arg: int) -> None:
        self._port = arg
